@@ -4,8 +4,8 @@ import numpy as np
 import typing
 
 from PySide6.QtCore import Signal, QObject
-from common.label_image import LabelImg
 
+from maphis.common.label_image import LabelImg
 from maphis.common.label_change import LabelChange, CommandEntry, DoType, CommandKind
 from maphis.common.photo import Photo, UpdateContext, UpdateEvent
 from maphis.common.state import State
