@@ -11,7 +11,7 @@ import cv2
 import numpy as np
 
 from PySide6 import QtGui, QtCore
-from PySide6.QtCore import Qt, Signal, QPoint, QObject, QRect
+from PySide6.QtCore import Qt, Signal, QPoint, QObject, QRect, QModelIndex
 from PySide6.QtGui import QImage, QPixmap
 from PySide6.QtWidgets import QWidget, QHBoxLayout, QLabel, QSpinBox, QPushButton, QComboBox, QDoubleSpinBox, QDialog, \
     QDialogButtonBox, QVBoxLayout, QStyleOptionViewItem, QGroupBox
@@ -19,14 +19,14 @@ from PySide6.QtWidgets import QWidget, QHBoxLayout, QLabel, QSpinBox, QPushButto
 from maphis import qimage2ndarray
 from maphis.common.blocking_operation import BlockingOperation
 from maphis.common.image_operation_binding import ImageOperation
-from maphis.common.photo import Photo, UpdateContext, UpdateEvent, PhotoUpdate, PhotoUpdateType
-from maphis.common.photo_layer import PhotoLayer
+from maphis.common.photo import Photo, UpdateEvent, PhotoUpdate, PhotoUpdateType
+from maphis.layers.photo_layer import PhotoLayer
 from maphis.common.state import State
 from maphis.common.storage import Storage, StorageUpdate
 from maphis.common.units import SIPrefix, Unit, BaseUnit, Value, UnitStore
 from maphis.common.utils import ScaleSetting, ScaleLineInfo, get_scale_marker_roi, get_reference_length, \
     get_scale_line_ends
-from maphis.common.visualization_layer import VisualizationLayer
+from maphis.layers.visualization_layer import VisualizationLayer
 from maphis.image_viewer import ImageViewer
 from maphis.thumbnail_storage import ThumbnailDelegate, ThumbnailStorage_
 from maphis.tools.ruler import Tool_Ruler
@@ -67,11 +67,6 @@ class ScaleSettingWidget(QWidget):
         self.units = UnitStore()
         self.line_length: typing.Optional[Value] = None
 
-        # self.setLayout(self._layout)
-        # self.line_length: Value = Value(0, self.state.units.units['px'])
-        # self.ref_length: Value = Value(0, self._cmbUnits.itemData(self._cmbUnits.currentIndex()))
-        # self.scale: Value = Value(0, self.state.units.units['px'] / self.ref_length.unit)
-
         self._setup_controls()
         self._setup_viewer()
 
@@ -87,7 +82,6 @@ class ScaleSettingWidget(QWidget):
         self._ref_label = QLabel("Reference length: ")
 
         self._spboxReference_mm = QSpinBox()
-        # self._spboxReference_mm.setSuffix(' mm')
         self._spboxReference_mm.setMinimum(1)
         self._spboxReference_mm.setMaximum(9999)
         self._spboxReference_mm.setMaximumWidth(150)
@@ -115,10 +109,10 @@ class ScaleSettingWidget(QWidget):
         self._groupAutoScaleExtraction.setLayout(QHBoxLayout())
 
         self._btnExtractScalesAllPhotos = QPushButton(text="Apply to all photos")
-        self._btnExtractScalesAllPhotos.clicked.connect(self._extract_scale_for_all)
+        self._btnExtractScalesAllPhotos.clicked.connect(lambda _: self.extract_scale_for_all())
 
         self._btnExtractScalesCurrentPhoto = QPushButton(text="Apply to this photo")
-        self._btnExtractScalesCurrentPhoto.clicked.connect(self._extract_scale_for_current)
+        self._btnExtractScalesCurrentPhoto.clicked.connect(lambda _: self.extract_scale_for_current())
 
         self._groupAutoScaleExtraction.layout().addWidget(self._btnExtractScalesCurrentPhoto)
         self._groupAutoScaleExtraction.layout().addWidget(self._btnExtractScalesAllPhotos)
@@ -127,11 +121,11 @@ class ScaleSettingWidget(QWidget):
         self._btnSetManually.clicked.connect(self._get_scale_from_user)
 
         self._btnAcceptAll = QPushButton(text="Accept changes for all photos")
-        self._btnAcceptAll.clicked.connect(self._accept_changes_all)
+        self._btnAcceptAll.clicked.connect(self.accept_changes_all)
         self._btnAcceptAll.setEnabled(True)
 
         self._btnAcceptCurrent = QPushButton(text="Accept changes for this photo")
-        self._btnAcceptCurrent.clicked.connect(self._accept_changes_current)
+        self._btnAcceptCurrent.clicked.connect(self.accept_changes_current)
         self._btnAcceptCurrent.setEnabled(True)
 
         self._btnResolutionGlobal = QPushButton()
@@ -178,9 +172,6 @@ class ScaleSettingWidget(QWidget):
         self.image_viewer.ui.tbtnRotateCW.hide()
         self.image_viewer.ui.tbtnRotateCCW.hide()
 
-        #self.image_viewer.rotate_cw_requested.connect(self._rotate_cw)
-        #self.image_viewer.rotate_ccw_requested.connect(self._rotate_ccw)
-
         self.image_viewer.rotate_cw_requested.connect(lambda: self.image_op.rotate(clockwise=True))
         self.image_viewer.rotate_ccw_requested.connect(lambda: self.image_op.rotate(clockwise=False))
 
@@ -198,7 +189,6 @@ class ScaleSettingWidget(QWidget):
 
         self.image_viewer.hide()
 
-        # self.ruler_tool: typing.Optional[Tool_Ruler] = None  # Tool_Ruler(self._state)
         self.ruler_tool: Tool_Ruler = Tool_Ruler(self._state)
         self.ruler_tool.show_real_units(False)
         self.ruler_tool.current_value.connect(self.compute_resolution)
@@ -210,74 +200,32 @@ class ScaleSettingWidget(QWidget):
         self.setLayout(self.image_viewer.layout())
 
     def _fetch_photo(self, idx):
-        idx = min(max(idx, 0), self._storage.image_count - 1)
-        # TODO: Is there any difference between self.temp_storage and self._state.storage?
-        # _fetch_photo() now serves as a single image fetching function, but some
-        # of the functions it replaced seemed to use self.temp_storage, and some
-        # seemed to use self._state.storage in some places.
-        # if self.temp_storage == self._state.storage:
-        #     print('self.temp_storage == self._state.storage')
-        # else:
-        #     print('!!! self.temp_storage != self._state.storage')
-        # self.image_viewer.enable_navigation_buttons(idx) # TODO: or should this work with temp_storage? (in that case, pass the appropriate storage as a parameter of enable_navigation_buttons()?)
-        photo = self._storage.get_photo_by_idx(idx)
+        photo = self.state.image_list_model.fetch_photo(idx)
+
+        if photo is None:
+            return
 
         scale_set = self.scale_settings[photo.image_path]
 
         self.setWindowTitle(f'{photo.image_name} - Set scale')
 
-        # self.image_viewer.setWindowTitle(f"Please delineate scale line for {photo.image_name}")
         self.image_viewer.set_photo(photo, reset_view=True, reset_tool=False)
 
-        # self.ruler_tool = self.ruler_tools[idx]
-        # value_mm: Optional[int] = None
-        # dig_re = re.compile(r'([0-9]+)\s*([a-zA-Z]m)')
-        # if self.ruler_tool._measurement <= 0:
-        #     scale_marker, (left, top, width, height) = get_scale_marker_roi(photo.image)
-        #     p1x, p1y, p2x, p2y = get_scale_line_ends(scale_marker)
-        #     scale_text, scale_roi = get_reference_length(scale_marker)
-        #     print(scale_text)
-        #     if len(scale_text) > 0:
-        #         match = dig_re.match(scale_text)
-        #         for group in match.groups():
-        #             print(group)
-        #         print(scale_text)
-        #     self.ruler_tool.set_line(QPoint(p1x+left, p1y+top), QPoint(p2x+left, p2y+top), reset_others=True)
-
-        # self.set_photo(photo)
-
-
-        # res = self.temp_storage.image_scales[idx]
-        # res = self.temp_storage.get_photo_by_idx(idx).image_scale
-        # if res is not None and res.value > 0:
-        #     self._lblResolution.setText(f'Scale: {res}')
-        # else:
-        #     self._lblResolution.setText('Scale: n/a')
-        #
-        # if self.ruler_tool.value_storage is not None and len(self.ruler_tool.value_storage) > 0:
-        #     self._lblPixels.setText(f'Line length: {self.ruler_tool.value_storage[0]} px')
-        # else:
-        #     self._lblPixels.setText('Line length: 0 px')
-
-        # self._scale_set_widget.set_photo(photo)
-
-        # if self.ruler_tool.value_storage is not None and len(self.ruler_tool.value_storage) > 0:
-        #     # self._scale_set_widget.set_line_length(Value(self.ruler_tool.value_storage[0], self._state.units.units['px']))
-        #     self.set_line_length(self.ruler_tool.value_storage[0])
-        # else:
-        #     self.set_line_length(Value(0, self._state.units.units['px']))
-
     def _fetch_first_photo(self):
         self._fetch_photo(0)
 
     def _fetch_prev_photo(self):
-        self._fetch_photo(self._state.storage.image_names.index(self._state.current_photo.image_name) - 1) # TODO: or should this work with temp_storage?
+        paths = self.state.image_list_model.image_paths
+        curr_idx_int = paths.index(self.state.current_photo.image_path)
+        self._fetch_photo(curr_idx_int - 1)
 
     def _fetch_next_photo(self):
-        self._fetch_photo(self._state.storage.image_names.index(self._state.current_photo.image_name) + 1) # TODO: or should this work with temp_storage?
+        paths = self.state.image_list_model.image_paths
+        curr_idx_int = paths.index(self.state.current_photo.image_path)
+        self._fetch_photo(curr_idx_int + 1)
 
     def _fetch_last_photo(self):
-        self._fetch_photo(self._state.storage.image_count - 1) # TODO: or should this work with temp_storage?
+        self._fetch_photo(self.state.image_list_model.rowCount(QModelIndex()) - 1)
 
     def _new_reference_length(self, value: int):
         self.compute_resolution(self.scale_settings[self.photo.image_path].new_scale_set.scale_line.length)
@@ -286,10 +234,6 @@ class ScaleSettingWidget(QWidget):
         idx = self.state.storage.image_names.index(self.state.current_photo.image_name)
         photo_ = self.state.storage.get_photo_by_idx(idx, load_image=False)
 
-        # for i in range(self.state.storage.image_count):
-        #     photo = self.state.storage.get_photo_by_idx(i, load_image=False)
-        #     photo.image_scale = photo_.image_scale
-
         scale_setting = self.scale_settings[photo_.image_path].new_scale_set
 
         for img_path, scale_tup in self.scale_settings.items():
@@ -299,13 +243,10 @@ class ScaleSettingWidget(QWidget):
             scale_tup.new_scale_set.scale_line = None #ScaleLineInfo((0, 0), (0, 0), Value(0, self.ruler_tool._px_unit))
             scale_tup.new_scale_set.reference_length = None
 
-        # self.scale_set.emit([i for i in range(self.state.storage.image_count)])
-        self.accepted.emit()
-
-    def _accept_changes_current(self):
+    def accept_changes_current(self):
         self.accepted.emit([self.state.current_photo])
 
-    def _accept_changes_all(self):
+    def accept_changes_all(self):
         self.accepted.emit(self.state.storage.images)
 
     def compute_resolution(self, pixels: typing.Optional[Value]):
@@ -326,9 +267,6 @@ class ScaleSettingWidget(QWidget):
         photo = self.state.storage.get_photo_by_idx(idx, load_image=False)
         ref_length = Value(self._spboxReference_mm.value(), self._cmbUnits.itemData(self._cmbUnits.currentIndex()))
         res = pixels / ref_length
-        # if photo.image_scale is None:
-        #     unit = self.state.units.units['px'] / self._cmbUnits.itemData(self._cmbUnits.currentIndex())
-        #     photo.image_scale = Value(value=res, unit=unit)
         self.scale = res
         scale_setting = self.scale_settings[photo.image_path]
         scale_setting.new_scale_set.scale = res
@@ -355,16 +293,10 @@ class ScaleSettingWidget(QWidget):
             scale_setting.new_scale_set.scale_line.p1 = self.ruler_tool.endpoints[0].toTuple()
             scale_setting.new_scale_set.scale_line.p2 = self.ruler_tool.endpoints[1].toTuple()
 
-        # photo.image_scale = res
-        # photo.image_scale.unit = self.state.units.units['px'] / self._cmbUnits.itemData(self._cmbUnits.currentIndex())
-        # self._lblResolution.setText(f'Scale: {scale_setting.new_scale_set.scale}')
-        # self._lblScaleValue.setText(str(scale_setting.new_scale_set.scale))
         self._set_lbl_scale_value_text(str(scale_setting.new_scale_set.scale))
-        # self._lblScaleValue.setSuffix(f' {res.unit}')
         self._btnAcceptAll.setEnabled(True)
         self._btnResolutionGlobal.setEnabled(True)
         self.set_line(scale_setting.new_scale_set.scale_line, scale_setting.new_scale_set.reference_length)
-        # self.scale_set.emit([idx])
 
     def set_photo(self, photo: Photo):
         self.photo = photo
@@ -372,42 +304,18 @@ class ScaleSettingWidget(QWidget):
         self.image_op.init(photo)
         self._state.current_photo = photo
 
-        res = photo.scale_setting.scale
         scale_setting = self.scale_settings[photo.image_path].new_scale_set
         self.ruler_tool.set_scale(scale_setting.scale)
         scale_line = scale_setting.scale_line
+        self.unset_line()
         if scale_line is not None:
-            # self.ruler_tool.set_line(QPoint(*scale_line.p1),
-            #                          QPoint(*scale_line.p2),
-            #                          reset_others=True)
-            # self.set_line_length(scale_line.length)
-            # self._spboxReference_mm.setEnabled(True)
-            # self._cmbUnits.setEnabled(True)
-            # if (ref_length := scale_setting.reference_length) is not None:
-            #     self._spboxReference_mm.setValue(ref_length.value)
-            #     self._cmbUnits.setCurrentIndex(list(SIPrefix).index(ref_length.unit.prefix))
             self.set_line(scale_line, scale_setting.reference_length)
         else:
             self.unset_line()
-            # self.ruler_tool.reset_tool()
-            # self._lblPixels.setText('Line length: no line drawn')
-            # # if there is not scale line yet, disable referene length spinner and units combobox
-            # # self.set_line_length(Value(0, self.state.units.units['px']))
-            # self._spboxReference_mm.setEnabled(False)
-            # self._cmbUnits.setEnabled(False)
-        if res is not None and res.value > 0:
-            # self._lblResolution.setText(f'Scale: {scale_setting.scale}')
-            # self._lblScaleValue.setText(str(scale_setting.scale))
+        if scale_setting.scale is not None and scale_setting.scale.value > 0:
             self._set_lbl_scale_value_text(str(scale_setting.scale))
-            # self._lblScaleValue.setSuffix(f' {scale_setting.scale.unit}')
-            # idx = list(SIPrefix).index(scale_setting.reference_length.unit.prefix)
-            # self._cmbUnits.setCurrentIndex(idx)
-            # ref_length = scale_setting.reference_length
-            # self._spboxReference_mm.setValue(ref_length.value)
         else:
             self._set_lbl_scale_value_text('not set')
-            # self._lblResolution.setText('Scale')
-            # self._lblScaleValue.setText('not set')
         self.setWindowTitle(f'{photo.image_name} - Set scale')
         self.image_viewer.set_tool(self.ruler_tool, reset_current=False)
         self.viz_layer.paint_commands = self.ruler_tool.viz_commands
@@ -427,9 +335,7 @@ class ScaleSettingWidget(QWidget):
         self.compute_resolution(self.line_length)
 
     def _handle_reference_unit_changed(self, idx: int):
-        # self._lblScaleValue.setSuffix(f' px / {self._cmbUnits.itemData(idx)}')
-        self.compute_resolution(self.line_length)
-        # self._lblScaleValue.setText(str(self.scale_settings[self.photo.image_path].new_scale_set.scale))
+        self.compute_resolution(self.scale_settings[self.photo.image_path].new_scale_set.scale_line.length)
         self._set_lbl_scale_value_text(str(self.scale_settings[self.photo.image_path].new_scale_set.scale))
 
     def initialize(self, storage: Storage):
@@ -449,11 +355,9 @@ class ScaleSettingWidget(QWidget):
                 new_scale_set=copy.deepcopy(photo.scale_setting)
             )
 
-    # def _handle_update_photo(self, img_name: str, ctx: UpdateContext, data):
     def _handle_update_photo(self, update: UpdateEvent):
         if self.photo is None or update.photo.image_name != self.photo.image_name:
             return
-        # photo = self._storage.get_photo_by_name(img_name)
         scale_setting = self.scale_settings[update.photo.image_path].new_scale_set
         self.ruler_tool.set_scale(scale_setting.scale)
         scale_line = scale_setting.scale_line
@@ -462,27 +366,12 @@ class ScaleSettingWidget(QWidget):
             self._cmbUnits.setCurrentIndex(list(SIPrefix).index(ref_length.unit.prefix))
         event_obj: PhotoUpdate = update.update_obj
         if scale_line is not None: #and 'operation' in data:
-            # if data['operation'].startswith('rot'):
             if event_obj.update_type in {PhotoUpdateType.Rotate90CCW, PhotoUpdateType.Rotate90CW}:
                 scale_line.rotate(event_obj.update_type == PhotoUpdateType.Rotate90CCW,
                                   (round(0.5 * update.photo.image_size[1]),
                                     round(0.5 * update.photo.image_size[0])))
-            # elif data['operation'] == Photo:
-            #     f = data['factor']
-            #     # scale_line.p1 = (round(f * scale_line.p1[0]),
-            #     #                  round(f * scale_line.p1[1]))
-            #     # scale_line.p2 = (round(f * scale_line.p2[0]),
-            #     #                  round(f * scale_line.p2[1]))
-            #     # scale_line.scale(f, (round(0.5 * photo.image_size[0]),
-            #     #                      round(0.5 * photo.image_size[1])))
-            #     pass
-            # self.ruler_tool.set_line(QPoint(*scale_line.p1),
-            #                          QPoint(*scale_line.p2),
-            #                          reset_others=True)
-            # self.set_line_length(scale_line.length)
             self.set_line(scale_line, scale_setting.reference_length)
         else:
-            # self.set_line_length(Value(0, self.state.units.units['px']))
             self.unset_line()
         self.viz_layer.paint_commands = self.ruler_tool.viz_commands
         self.viz_layer.update()
@@ -507,9 +396,6 @@ class ScaleSettingWidget(QWidget):
         self._cmbUnits.setEnabled(True)
         self._set_unit_to_cmbUnits(ref_length.unit)
         self.ruler_tool.set_line(QPoint(*scale_line.p1), QPoint(*scale_line.p2), reset_others=True)
-        # self.set_line_length(scale_line.length)
-        # self._lblScaleValue.setValue((scale_line.length / ref_length).value)
-        # self._lblScaleValue.setText(str(scale_line.length / ref_length))
         self._set_lbl_scale_value_text(str(scale_line.length / ref_length))
         self._lblPixels.setText(f'Line length: {scale_line.length}')
         self.viz_layer.paint_commands = self.ruler_tool.viz_commands
@@ -519,7 +405,6 @@ class ScaleSettingWidget(QWidget):
         self._spboxReference_mm.setEnabled(False)
         self._cmbUnits.setEnabled(False)
         self._lblPixels.setText('Line length: no line drawn')
-        # self._lblScaleValue.setText('not set')
         self._set_lbl_scale_value_text('not set')
         self.ruler_tool.reset_tool()
 
@@ -564,22 +449,29 @@ class ScaleSettingWidget(QWidget):
             scale_setting.scale_line = None
             scale_setting.reference_length = None
             self.unset_line()
-            # self._lblScaleValue.setText(str(scale_setting.scale))
             self._set_lbl_scale_value_text(str(scale_setting.scale))
 
-    def _extract_scale_for_current(self):
+    def extract_scale_for_current(self, parent: QWidget=None):
         idxs = [self.state.storage.image_names.index(self.state.current_photo.image_name)]
         block_op = BlockingOperation(self.state.storage, idxs,
                                      operation=self._scale_extraction_operation,
                                      result_handler=self._scale_extraction_result_processing,
-                                     parent=self)
+                                     parent=self if parent is None else parent)
         block_op.start()
 
-    def _extract_scale_for_all(self):
+    def extract_scale_for_all(self, parent: QWidget=None):
         block_op = BlockingOperation(self.state.storage, list(range(self.state.storage.image_count)),
                                      operation=self._scale_extraction_operation,
                                      result_handler=self._scale_extraction_result_processing,
-                                     parent=self)
+                                     parent=self if parent is None else parent)
+        block_op.start()
+
+    def extract_scale_for_photo(self, photo: Photo, parent: QWidget=None):
+        idx = self.state.storage.image_names.index(photo.image_name)
+        block_op = BlockingOperation(self.state.storage, [idx],
+                                     operation=self._scale_extraction_operation,
+                                     result_handler=self._scale_extraction_result_processing,
+                                     parent=self if parent is None else parent)
         block_op.start()
 
     def _scale_extraction_operation(self, storage: Storage, idx: int) -> typing.Optional[ScaleExtractionResult]:
@@ -611,7 +503,6 @@ class ScaleSettingWidget(QWidget):
         scale_rotated = cv2.line(scale_rotated, (p1x_, p1y_ - 10), (p1x_, p1y_ + 10), [0, 255, 0], thickness=2)
         scale_rotated = cv2.line(scale_rotated, (p2x_, p1y_ - 10), (p2x_, p1y_ + 10), [0, 255, 0], thickness=2)
         scale_rotated = cv2.resize(scale_rotated, (154, 26), interpolation=cv2.INTER_LINEAR)
-        # pixmap = QPixmap.fromImage(qimage2ndarray.array2qimage(scale_rotated))
         scale_marker = scale_rotated
 
         return ScaleExtractionResult(idx, image_scale, ref_length, px_length,
@@ -635,7 +526,7 @@ class ScaleSettingWidget(QWidget):
         sc_info.scale_marker_bbox = result.scale_marker_bbox
         sc_info.scale_marker_img = QPixmap.fromImage(qimage2ndarray.array2qimage(result.scale_marker))
 
-        if path == self.state.current_photo.image_path:
+        if self.state.current_photo is not None and path == self.state.current_photo.image_path:
             self.set_line(sc_info.scale_line, sc_info.reference_length)
         self._btnAcceptAll.setEnabled(True)
 
@@ -644,7 +535,6 @@ class ScaleSettingWidget(QWidget):
         self.cancelled.emit()
 
     def _set_lbl_scale_value_text(self, text: str):
-        # self._lblScaleValue.setText(f"<a href='scale_value'>{text}</a>")
         self._lblScaleValue.setText(text)
 
     def resizeEvent(self, event: PySide6.QtGui.QResizeEvent) -> None:
@@ -669,6 +559,5 @@ class ScaleItemDelegate(ThumbnailDelegate):
                            scale_marker.width(), scale_marker.height())
         painter.save()
         painter.setRenderHint(painter.SmoothPixmapTransform, True)
-        # painter.drawImage(scale_rect, scale_marker)
         painter.drawPixmap(scale_rect, scale_marker)
         painter.restore()
