@@ -24,6 +24,7 @@ from PySide6.QtWidgets import QApplication, QDialog, QDialogButtonBox, QComplete
     QStyledItemDelegate, QWidget, QStyleOptionViewItem, QLineEdit, QComboBox, QLabel, QHeaderView, QVBoxLayout, \
     QTableView
 
+from maphis.image_list_model import ImageListModel, ImageListSortFilterProxyModel
 from maphis import qimage2ndarray
 from maphis.common.image_operation_binding import ResizeWidget
 from maphis.common.label_image import LabelImgType
@@ -216,6 +217,10 @@ class ImportDialog(QDialog):
 
         self._scale_set_widget = ScaleSettingWidget(self._state)
         self._scale_set_widget.accepted.connect(self._handle_scales_accepted)
+        self._scale_set_widget.cancelled.connect(self._handle_scales_cancelled)
+
+    def _handle_scales_cancelled(self):
+        self._scale_set_widget.close()
 
     def _handle_scale_set(self, idxs: List[int]):
         first = self.image_list_model.index(idxs[0], 0, QModelIndex())
@@ -281,9 +286,17 @@ class ImportDialog(QDialog):
                 self.image_list_model.dataChanged.emit(self.image_list_model.index(index.row(), COL_DST_SIZE),
                                                   self.image_list_model.index(index.row(), COL_SCALE))
         elif index.column() == COL_SCALE:
+            self._scale_set_widget.state = self._state
+            self.image_list_model2 = ImageListModel()
+            self.image_list_model2.initialize(self._state.storage.image_paths, self.thumbnail_storage, 0, self._state.storage)
+            proxy = ImageListSortFilterProxyModel()
+            proxy.setSourceModel(self.image_list_model2)
+            self._state.image_list_model = proxy
+
             self._scale_set_widget.initialize(self.temp_storage)
             self._scale_set_widget.showMaximized()
-            photo = self.temp_storage.get_photo_by_idx(index.row(), True)
+            # photo = self.temp_storage.get_photo_by_idx(index.row(), True)
+            photo = self._state.image_list_model.fetch_photo(index.row())
             self._scale_set_widget.image_viewer.set_photo(photo, True)
             # self._scale_set_widget._fetch_photo(index.row())
 
@@ -482,6 +495,7 @@ class ImportDialog(QDialog):
             self.thumbnail_storage = ThumbnailStorage_(self.temp_storage, thumbnail_size=(128, 64))
 
             self.image_list_model.set_storage(self.temp_storage)
+            self._scale_set_widget.initialize(self.temp_storage)
             self.ui.imageList.reset()
             self.ui.imageList.setModel(self.image_list_model)
             self.ui.imageList.verticalHeader().setDefaultSectionSize(self.thumbnail_storage.thumbnail_size[1] + 32)
@@ -676,6 +690,9 @@ class ImportDialog(QDialog):
         self._update_import_button_text()
 
     def _handle_extract_scale_clicked(self):
+        self._scale_set_widget.extract_scale_for_all(self)
+        self._scale_set_widget.accept_changes_all()
+        return
         if self._scale_recognizing_in_progress:
             self._cancel_scale_extraction()
             self._scale_recognizing_in_progress = False
@@ -1057,7 +1074,7 @@ class ImageImportTableModel(QAbstractTableModel):
         elif index.column() == COL_SCALE_MARKER:
             if role == Qt.DecorationRole:
                 # return self.storage.get_photo_by_idx(index.row(), load_image=False).scale_marker
-                return photo.import_info.scale_marker
+                return photo.scale_setting.scale_marker_img
             elif role == Qt.SizeHintRole:
                 return QSize(154, 26)
         elif index.column() == COL_REF_LENGTH:
