@@ -8,8 +8,8 @@ from openpyxl.styles import Color, PatternFill, Border
 from openpyxl.styles.differential import DifferentialStyle
 from openpyxl.formatting.rule import CellIsRule, Rule
 from PySide6.QtWidgets import QApplication
-from common.utils import attempt_to_save_with_retries
 
+from maphis.common.utils import attempt_to_save_with_retries
 from maphis.common.common import Info
 from maphis.common.plugin import GeneralAction, ActionContext
 from maphis.common.state import State
