@@ -32,6 +32,7 @@ class Tool_Knife(Tool):
             self._tool_icon = QIcon(str(path))
         self.pen = QPen()
         self.state = state
+        self._current_cut_line_points: typing.Tuple[QPoint, QPoint] = (QPoint(), QPoint())
 
     @property
     def tool_name(self) -> str:
@@ -117,7 +118,11 @@ class Tool_Knife(Tool):
         return self.viz_mouse_move(pos, pos, canvas)
 
     def viz_mouse_move(self, new_pos: QPoint, old_pos: QPoint, canvas: QImage) -> List[PaintCommand]:
-        return [line_command(QPoint(*self._first_endpoint[::-1]), new_pos, self.pen)]
+        self._current_cut_line_points = (QPoint(*self._first_endpoint[::-1]), new_pos)
+        return [line_command(self._current_cut_line_points[0], self._current_cut_line_points[1], self.pen)]
+
+    def viz_mouse_wheel(self, delta: int, pos: QPoint, canvas: QImage) -> List[PaintCommand]:
+        return self.viz_mouse_move(self._current_cut_line_points[1], QPoint(), canvas)
 
     def viz_left_release(self, pos: QPoint, canvas: QImage) -> List[PaintCommand]:
         self._active = False
@@ -129,9 +134,12 @@ class Tool_Knife(Tool):
         curr_width = self._width_param.value
         new_width = max(1, curr_width + my_delta)
         self._width_param.set_value(new_width)
+        self.pen.setWidth(self._width_param.value)
         # self.cursor_changed.emit(self.tool_id, self.cursor_image)
         return None, QRect()
 
+
+
     @property
     def setting_widget(self) -> typing.Optional[QWidget]:
         return self._width_param_widget.widget
